package at.jku.risc.stout.hoau;

import static at.jku.risc.stout.hoau.algo.DebugLevel.SILENT;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.util.Map;
import java.util.Set;

import at.jku.risc.stout.hoau.algo.AntiUnify;
import at.jku.risc.stout.hoau.algo.AntiUnifyProblem;
import at.jku.risc.stout.hoau.algo.PermEquiv;
import at.jku.risc.stout.hoau.algo.PermEquivProblem;
import at.jku.risc.stout.hoau.data.Equation;
import at.jku.risc.stout.hoau.data.EquationSystem;
import at.jku.risc.stout.hoau.data.InputParser;
import at.jku.risc.stout.hoau.data.TermNode;
import at.jku.risc.stout.hoau.data.atom.Variable;
import at.jku.risc.stout.hoau.util.ControlledException;

public class LambdaTermPair {

	int maxReduce = 100; // maximum number of recursions for beta reduction
	boolean justify = false; // justify computed result
	Reader term1;
	Reader term2;

	private <T extends Equation> EquationSystem<T> parseEqSys(
			final Class<T> template) throws ControlledException, IOException {
		EquationSystem<T> eqSys = new EquationSystem<T>() {
			public T newEquation() {
				try {
					return template.newInstance();
				} catch (Exception e) {
					throw new AssertionError("Illegal state", e);
				}
			}
		};
		new InputParser().parseEquation(term1, term2, eqSys, maxReduce);
		return eqSys;
	}

	public TermNode generalize() throws IOException, ControlledException {
		AntiUnify au = new AntiUnify(parseEqSys(AntiUnifyProblem.class), maxReduce, SILENT);
		au.antiUnify(justify, null);
		// The result is unique for two input terms.
		Variable genVar = au.getEqSysOrig().getFirst().getGeneralizationVar();
		return au.getResult().getSigma().get(genVar);
	}

	public Map<Variable, Variable> equivTest(Set<Variable> dom,
			Set<Variable> ran) throws IOException, ControlledException {
		return new PermEquiv(parseEqSys(PermEquivProblem.class), dom, ran).compute(SILENT, null);
	}

	public static void main(String... args) throws Exception {
		LambdaTermPair tp = new LambdaTermPair();
		tp.term1 = new StringReader("\\x.f(b,a,a)"); // Use FileReader instead
		tp.term2 = new StringReader("\\x.g(a,x,b)"); // Use FileReader instead
		System.out.println(tp.generalize());
	}
}
