/*
 * Copyright 2012 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.hoau.data.atom;

import java.io.IOException;
import java.io.Writer;

/**
 * A bound variable is special type of {@linkplain Variable} which stores it's
 * origin name because it may be renamed by alpha conversion.
 * 
 * @author Alexander Baumgartner
 */
public class BoundVariable extends Variable {
	public static boolean PRINT_ORIGIN_NAME = false;
	private String originName;

	public BoundVariable(String name, String originName, String type) {
		super(name, type);
		this.originName = originName.intern();
	}

	@Override
	public void print(Writer out) throws IOException {
		if (PRINT_ORIGIN_NAME)
			super.print(out, getOriginName());
		else
			super.print(out);
	}

	@Override
	public String getOriginName() {
		return originName;
	}
}
