/*
 * Copyright 2012 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.hoau.data.atom;

import java.io.IOException;
import java.io.Writer;

import at.jku.risc.stout.hoau.data.InputParser;

/**
 * A {@linkplain TermAtom} which represents a lambda binding. The binding
 * contains the bound variable (see {@linkplain #getBoundVar()})
 * 
 * @author Alexander Baumgartner
 */
public class Lambda extends TermAtom {
	private BoundVariable boundVar;

	public Lambda(BoundVariable boundVar) {
		super(String.valueOf(InputParser.LAMBDA_NAME));
		this.boundVar = boundVar;
	}

	@Override
	public TermAtom clone() {
		Lambda clone = (Lambda) super.clone();
		clone.boundVar = (BoundVariable) boundVar.clone();
		return clone;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof Lambda) {
			Lambda lambda = (Lambda) obj;
			return super.equals(lambda)
					&& boundVar.getType() == lambda.boundVar.getType();
		}
		return false;
	}

	@Override
	public String getType() {
		return boundVar.getType();
	}

	@Override
	public boolean isBasicType() {
		return false;
	}

	/**
	 * Returns the variable which is bound by this lambda binding.
	 */
	public BoundVariable getBoundVar() {
		return boundVar;
	}

	@Override
	public void print(Writer out) throws IOException {
		out.append(getName());
		boundVar.print(out);
		out.append('.');
	}
}
