/*
 * Copyright 2012 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.hoau.data.atom;

import at.jku.risc.stout.hoau.util.Printable;

/**
 * Base class for all the atomic types like function symbols and variables.
 * 
 * @author Alexander Baumgartner
 */
public abstract class TermAtom extends Printable implements
		Comparable<TermAtom>, Cloneable {
	protected String name;
	private Class<? extends TermAtom> clazz = getClass();

	public abstract String getType();

	public abstract boolean isBasicType();

	public TermAtom(String name) {
		this.name = name.intern();
	}

	public String getName() {
		return name;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof TermAtom) {
			TermAtom atom = (TermAtom) obj;
			// We only need to compare the pointers, because of calling intern
			// at construction time
			return name == atom.name && clazz == atom.clazz
					&& getType() == atom.getType();
		}
		return false;
	}

	@Override
	public TermAtom clone() {
		try {
			return (TermAtom) super.clone();
		} catch (CloneNotSupportedException e) {
			throw new RuntimeException(e);
		}
	}

	@Override
	public int hashCode() {
		return name.hashCode();
	}

	@Override
	public int compareTo(TermAtom o) {
		String n1 = this.name, n2 = o.name;
		if (n1 == n2) {
			Class<? extends TermAtom> c1 = clazz;
			Class<? extends TermAtom> c2 = o.clazz;
			if (c1 == c2)
				return 0;
			return c1.getSimpleName().compareTo(c2.getSimpleName());
		}
		return n1.compareTo(n2);
	}

	public String getOriginName() {
		return getName();
	}

	public String appendType(String type, char typeConstructor) {
		if (getType() == null)
			return type;
		if (type.length() == 0)
			return getType();
		return type + typeConstructor + getType();
	}
}
